/**
 * @file    下载代码脚本
 * @module  pull.js
 * @author  r2space@gmail.com
 * @version 1.0.0
 */

'use strict';

const fs      = require('fs');
const path    = require('path');
const debug   = require('debug')('light:bin');
const helper  = require('../lib/helper');
const Context = require('../lib/context');
const CONST   = require('../lib/constant');
const light   = require('../lib/model/light');


class Pull {

  async start(root = '/data') {

    const domain = process.env.APPNAME;

    debug('domain', domain);
    const handler = new Context(CONST.DEFAULT_USER, domain, CONST.SYSTEM_DB_PREFIX);
    const condition = {
      select   : 'name, md5, type, source',
      condition: { lang: 'node' }
    };
    const result = await light.code.list(handler, condition);

    for (const code of result.items) {

      const isTest = process.env.APPDEV === 'true' || helper.isUnitTest();
      const name = isTest
        ? path.join(process.cwd(), '/tmp/data', domain, code.name)
        : path.join(root, domain, code.name);

      helper.mkdirp(name, true);
      if (fs.existsSync(name) && helper.fileMd5(name) === code.md5) {
        continue;
      }

      debug(`>> update ${code.name}`);

      if (code.type === 'binary') {
        await this.writeBinaryFile(handler, name, code.source);
      }

      if (code.type === 'code') {
        await this.writeCodeFile(name, code.source);
      }
    }

  }

  writeBinaryFile(handler, path, source) {
    return new Promise((resolve, reject) => {
      handler.params.id = source;
      helper.stream(handler)
        .then(stream => {
          stream.pipe(fs.createWriteStream(path));
          stream.on('end', () => resolve());
        })
        .catch(err => reject(err));
    });
  }

  writeCodeFile(path, source) {
    return new Promise((resolve, reject) => {
      fs.writeFile(path, source, 'utf8', err => {
        if (err) {
          return reject(err);
        }
        resolve();
      });
    });
  }

}

if (!module.parent) {
  const Client  = require('../client');
  Client.init('client')
    .then(async () => {
      const pull = new Pull();
      await pull.start();
    })
    .catch(err => console.error(err))
    .finally(async () => {
      await Client.destory();
      process.exit();
    });
}

module.exports = Pull;
