/**
 * 从数据库configuration表中读取配置项
 * - 服务器启动时调用一次load，更新配置项需要重新启动服务
 * - 支持 string，number，boolean，array 类型
 * - 对 number，boolean 会进行类型转换
 * @module lib/configuration
 * @author r2space@gmail.com
 * @version 1.0.0
 */

'use strict';

const assert = require('assert').strict;
const helper = require('./helper');

class Configuration {

  static catalog = [];

  static load(catalog) {

    assert(catalog, 'catalog cannot be empty');

    this.catalog = [].concat(this.catalog).concat(catalog);

    catalog.forEach(item => {

      const { type, key } = item;

      assert(type, '[type] cannot be empty');
      assert(key, '[key] cannot be empty');

      // 根据环境，获取不同的设定值
      const value = process.env.APPENV
        ? item.value[process.env.APPENV]
        : item.value;

      helper.set(this, `${type}.${key}`, this.parse(value, item.valueType));
    });

  }

  /**
   * 类型转换，支持三种类型
   *  string
   *  number
   *  boolean
   * @param {*} value
   * @param {*} type
   */
  static parse(value, type = 'string') {

    switch (type.toLowerCase()) {
      case 'string':
        return String(value);

      case 'number':
        if (typeof value === 'string') {
          const result =  Number(value);
          if (Number.isNaN(result)) {
            return value;
          }
          return result;
        }
        return value;

      case 'boolean':
        if (typeof value === 'string') {
          return value !== 'false' && value !== '0' && value !== '';
        }
        if (typeof value === 'number') {
          return value !== 0;
        }
        return value;

      default:
        return value;
    }
  }

}

module.exports = Configuration;
