/**
 * @file 常量定义类
 * @author r2space@gmail.com
 * @module light.core.constant
 * @version 1.0.0
 */

'use strict',

module.exports = {

  VALID  : 1,
  INVALID: 0,

  /** LOG的输出级别 */
  LOG_LEVEL_DEBUG: 3,
  LOG_LEVEL_WARN : 2,
  LOG_LEVEL_INFO : 1,
  LOG_LEVEL_ERROR: 0,

  /** LOG的种类 */
  LOG_TYPE_APPLICATION: 'application',
  LOG_TYPE_AUDIT      : 'audit',
  LOG_TYPE_OPERATION  : 'operation',

  PATH_ROUTE     : '/routes/',
  PATH_CONTROLLER: '/controllers/',
  PATH_MIDDLEWARE: '/middleware/',
  PATH_MODULES   : '/modules/',

  DB_DEFAULT_LIMIT: 0,          // 不再限制获取数据的大小，默认返回所有数据，大数据量的时候一定要设定limit参数
  DB_MILLION_LIMIT: 2147483647, // 最大整型常量
  DB_GRIDFS_CTYPE : 'application/octet-stream',

  SYSTEM_DB_HOST     : 'db',
  SYSTEM_DB_PORT     : '57017',
  SYSTEM_DB          : '8058e10b3db4',
  SYSTEM_DB_REPLICA  : 'light',
  SYSTEM_DB_PREFIX   : 'light',
  SYSTEM_DB_MECHANISM: 'SCRAM-SHA-1',
  SYSTEM_DB_CONFIG   : 'configuration',
  SYSTEM_DB_VALIDATOR: 'validator',
  SYSTEM_DB_I18N     : 'i18n',
  SYSTEM_DB_STRUCTURE: 'structure',
  SYSTEM_DB_BOARD    : 'board',
  SYSTEM_DB_ROUTE    : 'route',
  SYSTEM_DB_TENANT   : 'tenant',
  SYSTEM_DB_SETTING  : 'setting',
  SYSTEM_DB_FUNCTION : 'function',
  SYSTEM_DB_JOB      : 'job',
  SYSTEM_DB_MQ       : 'mq',

  DEFAULT_TENANT   : 'default',
  DEFAULT_USER_TYPE: 'app_admin',
  DEFAULT_USER     : '000000000000000000000001',

  COOKIE_SESSION_KEY: 'light.sid',
  COOKIE_CSRF_KEY   : 'light.csrf',
  COOKIE_LANG_KEY   : 'light.lang',
  SUPPORT_LANG      : ['zh', 'ja', 'en'],

  FUNC_STATUS_ENABLED : '1',
  FUNC_STATUS_DISABLED: '2',
  FUNC_KIND_DEVELOPER : 'developer',
  FUNC_KIND_OPERATION : 'operation',

  METHOD_GET   : 'GET',
  METHOD_LIST  : 'LIST',
  METHOD_ADD   : 'ADD',
  METHOD_UPDATE: 'UPDATE',
  METHOD_REMOVE: 'REMOVE',
  METHOD_SEARCH: 'SEARCH',
  METHOD_COUNT : 'COUNT',

  BOARD_TYPE : ['list', 'add', 'update', 'remove', 'list', 'search', 'get', 'count'],
  METHOD     : ['GET', 'POST', 'PUT', 'DELETE', 'GET', 'GET', 'GET', 'GET'],
  METHOD_NAME: ['', 'ADD', 'UPDATE', 'REMOVE', 'LIST', '', 'GET', 'COUNT'],

  JOB_COMPLETED: 'COMPLETED',
  JOB_FAILED   : 'FAILED',
  JOB_STARTED  : 'STARTED',
  JOB_PAUSED   : 'PAUSED',

  API_KIND_USER_DATA   : 0,            // 用户定义数据型API
  API_KIND_USER_LOGIC  : 1,            // 用户定义逻辑型API
  API_KIND_SYSTEM_DATA : 2,            // 系统定义数据型API
  API_KIND_SYSTEM_LOGIC: 3,            // 系统定义逻辑型API

  MQ: {
    STATUS_WAITING   : 'waiting',
    STATUS_FINISHED  : 'finished',
    STATUS_PROCESSING: 'processing',
    STATUS_FAILED    : 'failed',
    TIMEOUT          : 60 * 1000 * 5,   // 默认5分钟超时
    INTERVAL         : 3,               // 默认轮询间隔3秒

    PREFIX: 'light',

    HOST: 'rabbitmq',
    PORT: 5672,
    USER: 'root',
    PASS: 'alphabets'
  },
  APP_KIND_SERVER: 'server',
  APP_KIND_CLIENT: 'client',

  VALIDATOR: {
    CUSTOM: 'validator'
  },

  SESSION: {
    DOMAIN: 'domain',
    CODE  : 'code',
    USER  : 'user',
    CORP  : 'corp',
    DB    : 'db'
  },

  JWT: {
    TIMEZONE: 'timezone',
    ID      : '_id',
    DOMAIN  : 'domain',
    CODE    : 'code'
  },

  SYMBOL: {
    JWT   : Symbol.for('context#jwtdata'),
    OAUTH : Symbol.for('context#oauthdata'),
    QUERY : Symbol.for('context#request#query'),
    XML   : Symbol.for('context#request#xml'),
    DOMAIN: Symbol.for('context#domain'),
    CODE  : Symbol.for('context#code')
  },

  /**
   * 扩展校验
   */
  EXTEND: {
    RULE: { CHECKFIELD: 'checkField' }
  },

  /**
   * built-in角色
   */
  ROLE: {
    DEFAULT: 'default'
  },

  /**
   * built-in动作
   */
  ACTION: {
    ACCESS: 'ACCESS',
    READ  : 'READ',
    WRITE : 'WRITE'
  },

  /**
   * 文件表预留字段
   */
  FILE: {
    FIELD : ['length', 'md5', 'chunkSize', 'uploadDate', 'filename', 'contentType'],
    METHOD: ['add', 'update', 'remove']
  }

};
