/**
 * 设置当前日期
 *
 * {
 *  $currentDate: {
 *    field: true,
 *    field: { $type: 'timestamp' },
 *    field: { $type: 'date' },
 *  }
 * }
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator')
  , assert = require('assert').strict
  , type = require('../type');

exports.parse = function (value, define, addition) {

  debug('$currentDate parse', value, define);
  assert(type.isJson(value), 'invalid $currentDate value');

  Object.keys(value).forEach(key => {

    // 如果是Json，内容需满足特定格式 { $type: 'date' }
    if (type.isJson(value[key])) {

      const [itemKey] = Object.keys(value[key])
        , [itemVal] = Object.values(value[key]);

      // key 必须是 $type
      assert(itemKey === '$type', 'invalid $type');

      // value 需是timestamp或date
      return assert(['timestamp', 'date'].includes(itemVal), 'invalid $type value');
    }

    // 否则视为boolean
    value[key] = type.parse(value[key], { type: 'boolean' }, addition);
  });

  debug('$currentDate parsed', value);
  return value;
};

exports.compare = function (field, value) {
  return { $and: value };
};
