/**
 * 对数组文档，使用and条件检索
 *
 * {
 *  a: { $elemMatch: { $gte: 80, $lt: 85 } }
 *  b: { $elemMatch: { product: "xyz", score: { $gte: 8 } } } }
 * }
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator')
  , mapping = require('../mapping')
  , type = require('../type');

exports.parse = function (value, defines, addition) {

  debug('$elemMatch parse', value, defines);

  // defines不是单个值的定义，则直接使用defines
  // 如 { a: {type: 'number'}, b: {type: 'number'} }
  let define = defines;

  // 是单个值的定义，且有有contents，使用contents
  // 如 { type: 'array', contents: 'number' }
  if (defines.type && define.contents) {
    define = type.isJson(define.contents)
      ? define.contents
      : { type: define.contents };
  }

  const result = mapping.parse(value, define, addition);

  debug('$elemMatch parsed', result);
  return result;
};

exports.compare = function (field, value) {
  return { [field]: { $elemMatch: value } };
};
