/**
 * 指定的值是否在列表中
 *
 * {
 *  a: { $in: [1] },          -> a type is number
 *  b: { $in: [/^$/] },       -> b type is string
 *  c: { $in: [{ c: '1' }] }  -> c type is object
 *  d: { $in: [1] }           -> d type is array, contents: number
 *  e: { $in: [{ e: '1' }] }  -> e type is array, contents: object
 * }
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator')
  , mapping = require('../mapping')
  , type = require('../type');

exports.parse = function (value, defines, addition, kind = '$in') {

  debug(kind, 'parse', value, defines);

  // 值必须是数组
  let result = type.parse(value, { type: 'array' }, addition)
    , define = defines;

  // contents被指定，优先使用
  if (define.contents) {
    define = type.isJson(define.contents) ? define.contents : { type: define.contents };
  }

  result = result.map(item => (type.isJson(item)
    ? mapping.parse(item, define, addition)
    : mapping.parseItem(item, define, addition)));

  debug(kind, 'parsed', result);
  return result;
};

exports.compare = function (field, value) {

  // in比较，如果传递的是字符串，尝试转换成数组
  if (typeof value === 'string') {
    return { [field]: { $in: value.split(',').map(item => item.trim()) } };
  }

  return { [field]: { $in: value } };
};
