/**
 * 删除数组的中满足条件的元素
 *
 * {
 *   $pull: {
 *     num: 'light',
 *     fruits: { $in: [ 'apples', 'oranges' ] }
 *   }
 * }
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator')
  , assert = require('assert').strict
  , type = require('../type')
  , mapping = require('../mapping');

exports.parse = function (value, defines, addition, kind) {

  debug(kind, 'parse', value, defines);
  assert(type.isJson(value), `invalid ${kind} value`);

  Object.keys(value).forEach(key => {

    // 如果没有定义contents，就使用当前的type 如 { type: 'number' }
    // 如果contents是json就直接使用 如 { type: 'array', contents: { a: {type: 'number'}, b: {type: 'number'} } }
    // 如果contents是基本类型那将该类型视为type 如 { type: 'array',  contents: 'number' }
    let define = mapping.find(key, defines);
    if (define.contents) {
      define = type.isJson(define.contents)
        ? define.contents
        : { type: define.contents };
    }

    value[key] = type.isJson(value[key])
      ? mapping.parse(value[key], define, addition)
      : mapping.parseItem(value[key], define, addition);
  });

  debug(kind, 'parsed', value);
  return value;
};
