/**
 * 从数组删除指定值的元素，而pull是按条件删除
 *
 * {
 *  $pullAll: {
 *    tests: [1, 2, 3]
 *  }
 * }
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator')
  , assert = require('assert').strict
  , type = require('../type')
  , mapping = require('../mapping');

exports.parse = function (value, defines, addition, kind = '$pullAll') {

  debug(kind, 'parse', value, defines);
  assert(type.isJson(value), `invalid ${kind} value`);

  Object.keys(value).forEach(key => {

    let define = mapping.find(key, defines);
    if (define.contents) {
      define = type.isJson(define.contents)
        ? define.contents
        : { type: define.contents };
    }

    // 值必须是数组
    const array = type.parse(value[key], { type: 'array' }, addition);
    value[key] = array.map(item => (type.isJson(item)
      ? mapping.parse(item, define, addition)
      : mapping.parseItem(item, define, addition)));
  });

  debug(kind, 'parsed', value);
  return value;
};
