/**
 * 排序数组，与$each结合使用
 *
 * {
 *  $push: [
 *    a: {
 *      $each: [1, 2, 3],
 *      $sort: -1
 *    }
 *  ]
 * }
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator')
  , assert = require('assert').strict
  , type = require('../type');

exports.parse = function (value) {
  debug('$sort parse', value);

  // 可以是排序字段，值只能是 1 或 -1
  if (type.isJson(value)) {
    Object.keys(value).forEach(key => {
      value[key] = type.parse(value[key], { type: 'number' });
      assert([1, -1].includes(value[key]), 'invalid $sort value');
    });

    debug('$sort parsed', value);
    return value;
  }

  // 值只能是 1 或 -1
  const result = type.parse(value, { type: 'number' });
  assert([1, -1].includes(result), 'invalid $sort value');

  debug('$sort parsed', result);
  return result;
};
