/**
 * 查询字段类型匹配的数据
 * 值必须是正确的BSON类型（参看BSON类型说明）
 *
 * {
 *  a: { $type: 1 },
 *  b: { $type: ['string', 'number'] },
 * }
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator')
  , assert = require('assert').strict
  , TYPE = {
    STR: [
      'double', 'string', 'object', 'array', 'binData',
      'objectId', 'bool', 'date', 'null', 'regex',
      'javascript', 'javascriptWithScope', 'int',
      'timestamp', 'long', 'decimal', 'minKey', 'maxKey',
      'number' // number是double，decimal等的别名
    ],
    NUM: [
      1, 2, 3, 4, 5, 7, 8, 9, 10, 11,
      13, 15, 16, 17, 18, 19, -1, 127
    ]
  };

exports.parse = function (value) {

  debug('$type parse', value);

  (Array.isArray(value) ? value : [value]).forEach(item => {
    assert(
      TYPE.STR.includes(item) || TYPE.NUM.includes(item),
      'invalid $type value'
    );
  });

  debug('$type parsed', value);
  return value;
};

exports.compare = function (field, value) {
  return { [field]: { $type: value } };
};
