/**
 * Mongo操作符支持
 *  支持生成比较对象
 *  支持类型转换
 * @module operator/index.js
 * @author r2space@hotmail.com
 * @version 1.0.0
 */

'use strict';

const debug = require('debug')('light:db:mongo:operator');
const helper = require('../../../helper');

class Operator {

  /**
   * 将给定的操作符与键值对转换为检索用JSON对象
   *  1. 支持所有 mongodb 的 query selector
   *     https://docs.mongodb.com/manual/reference/operator/query/
   *  2. 这个转换，不对值进行校验，如in的值需要是数组等
   *  3. 平台特殊转换的操作符
   *     $in, $nin 支持字符串，将会会尝试用[,]进行分割
   *  4. 不支持的操作符
   *     $expr, $jsonSchema, $where
   *     $geoIntersects, $geoWithin, $near, $nearSphere
   *     $bitsAllClear, $bitsAllSet, $bitsAnyClear, $bitsAnySet
   *     $comment, $meta
   * @param operator
   * @param field
   * @param value
   * @returns {*}
   */
  static compare(operator, field, value) {

    const clazz = helper.resolve(operator, __dirname);
    if (!clazz) {
      return { [field]: { [operator]: value } };
    }

    return clazz.compare(field, value);
  }

  /**
   * 根据操作符及数据定义，对指定的数据进行类型转换
   * @param operator
   * @param value
   * @param define
   * @param addition
   * @returns {*}
   */
  static parse(operator, value, define, addition) {

    const clazz = helper.resolve(operator, __dirname);
    if (clazz) {
      return clazz.parse(value, define, addition);
    }

    debug('unsupported operator', operator);
    return value;
  }

}

module.exports = Operator;
