/**
 * type/index.js
 *  根据给定的类型，对值进行转换
 *  支持 array boolean date number object objectid string
 *  其他类型 原值返回
 * @author r2space@gmail.com
 * @module light.lib.mongo.type
 * @version 1.0.0
 */
'use strict';

const debug = require('debug')('light:db:mongo:type');
const assert = require('assert').strict;
const helper = require('../../../helper');

class Type {

  /**
   * 用指定的类型名进行类型转换
   * @param object
   * @param define
   * @param addition 转换参数，如timezone等
   * @returns {*}
   */
  static parse(object, define, addition) {

    debug('parse', object, 'to', define, addition || '-');
    assert(define.type, 'type cannot be empty');

    const clazz = helper.resolve(define.type.toLowerCase(), __dirname);

    debug('use %s for conversion', define.type);
    if (!clazz) {
      debug('can not convert, unsupported type', define.type);
      return object;
    }

    // 是基本类型的数组，逐项转换
    if (define.type === 'array' && define.contents && !this.isJson(define.contents)) {
      debug('is an array of basic types', define.contents);
      return clazz
        .parse(object, addition)
        .map(item => this.parse(item, { type: define.contents }, addition));
    }
    return clazz.parse(object, define, addition);
  }

  /**
   * 是否是空白
   *  null, undefined, NaN, Infinity, '', ' ', []
   * @param {*} value
   */
  static isBlank(value) {
    return helper.isBlank(value);
  }

  /**
   * 是否是json对象
   * @param {*} value
   */
  static isJson(value) {
    return helper.isJson(value);
  }

}

module.exports = Type;
