/**
 * Created by sh on 15/4/13.
 */

'use strict';

const debug = require('debug')('light:db:mongo:type');
const type = require('./index');

exports.parse = function (param, define = {}) {

  // 允许通过define，指定值为空时取值。默认是 {}
  let defaults = {};
  if (define.default === 'undefined') {
    defaults = undefined;
  }
  if (define.default === 'null') {
    defaults = null;
  }

  const value = param instanceof String ? param.valueOf() : param;
  if (type.isBlank(value)) {
    debug('is blank');

    // 如果是空，原值是空数组就返回空数组
    return Array.isArray(value) ? [] : defaults;
  }

  // 尝试解析json形式的字符串
  if (typeof value === 'string') {
    try {
      const parsed = JSON.parse(value);
      debug('json parsed', parsed);

      // 转换完后值是空值，直接返回
      if (type.isBlank(parsed)) {
        debug('is blank');

        // 如果是空，原值是空数组就返回空数组
        return Array.isArray(value) ? [] : defaults;
      }

      // 转换完后值是boolean和number，那么使用原值
      // JSON.parse('1') => 1，我们应该用原值 '1' 而不是数字 1
      if (typeof parsed === 'boolean' || typeof parsed === 'number') {
        return value;
      }

      return parsed;
    } catch (err) {
      // nothing
    }
  }

  return value;
};
