/**
 *
 */

const debug = require('debug')('light:db:sessionstore')
  , Model = require('./mongo/model')
  , CONST = require('../constant');

class SessionStore {

  static init(option) {
    return new SessionStore(option);
  }

  constructor(option) {

    const table = option.table || 'sessions';
    debug('init session', table);

    this.model = new Model(process.env.APPNAME, CONST.SYSTEM_DB_PREFIX, table);
  }

  async destroy(id) {
    debug('destroy session', id);
    return await this.model.remove({ id });
  }

  async get(id) {
    debug('get session', id);
    const { data } = await this.model.get({ id });
    return data;
  }

  async set(id, data, maxAge, { changed, rolling }) {

    debug('save session', id, changed, rolling);
    if (changed || rolling) {
      return await this.model.upsert({ id }, { id, data, updatedAt: new Date() });
    }

    return data;
  }

}

module.exports = SessionStore;
