/**
 * @file DB异常类。
 * @module light.core.error.db
 * @author qiou_kay@163.com
 * @version 1.0.0
 */


'use strict';

const { AbstractError } = require('./common');

/**
 * @desc DB异常类
 * @param {String} msg 错误信息
 */
class DBError extends AbstractError {

  constructor(msg) {
    super(msg || 'DB Error');
    this.code = 'D0000';
  }

}

/**
 * @desc 添加数据时的DB错误
 * @param {String} msg 错误信息
 */
class AddError extends DBError {

  constructor(msg) {
    super(msg || 'Failed to Add Error');
    this.code = 'D0001';
  }

}

/**
 * @desc 删除数据时的DB错误
 * @param {String} msg 错误信息
 */
class RemoveError extends DBError {

  constructor(msg) {
    super(msg || 'Failed to Remove Error');
    this.code = 'D0002';
  }

}

/**
 * @desc 更新数据时的DB错误
 * @param {String} msg 错误信息
 */
class UpdateError extends DBError {

  constructor(msg) {
    super(msg || 'Failed to Update Error');
    this.code = 'D0003';
  }

}

/**
 * @desc 检索数据时的DB错误
 * @param {String} msg 错误信息
 */
class FindError extends DBError {

  constructor(msg) {
    super(msg || 'Failed to Find');
    this.code = 'D0004';
  }

}

/**
 * @desc 数据不存在
 * @param {String} msg 错误信息
 */
class NotExistError extends DBError {

  constructor(msg) {
    super(msg || 'Not Exist Error');
    this.code = 'D0004';
  }

}

/**
 * @desc Collection被锁定
 * @param {String} msg 错误信息
 */
class LockedError extends DBError {

  constructor(msg) {
    super(msg || 'Collection is locked Error');
    this.code = 'D0005';
  }

}

/**
 * @desc 数据不正确
 * @param {String} msg 错误信息
 */
class NotCorrectError extends DBError {

  constructor(msg) {
    super(msg || 'Not Correct Error');
    this.code = 'D0006';
  }

}

/**
 * exports
 */
module.exports = {
  DBError,
  AddError,
  RemoveError,
  UpdateError,
  FindError,
  NotExistError,
  LockedError,
  NotCorrectError
};
