/**
 * @file 参数异常类。
 * @module light.core.error.parameter
 * @author qiou_kay@163.com
 * @version 1.0.0
 */

'use strict';

const { AbstractError } = require('./common');

/**
 * @desc 参数异常类
 * @param {String} msg 错误信息
 */
class ParamError extends AbstractError {

  constructor(msg) {
    super(msg || 'Param Error');

    this.code = 'P0000';
    this.status = 400;
  }

}

/**
 * @desc 密码异常类
 * @param {String} msg 错误信息
 */
class PasswordError extends AbstractError {

  constructor(msg) {
    super(msg || 'Account or Password Error');
    this.code = 'P0001';
    this.status = 400;
  }

}

/**
 * @desc 类型异常类
 * @param {String} msg 错误信息
 */
class TypeError extends AbstractError {

  constructor(msg) {
    super(msg || 'Type Error');
    this.code = 'P0002';
    this.status = 400;
  }

}

/**
 * @desc 参数为空异常类
 * @param {String} msg 错误信息
 */
class EmptyError extends AbstractError {

  constructor(msg) {
    super(msg || 'Empty Error');
    this.code = 'P0003';
    this.status = 400;
  }

}

class CaptchaError extends AbstractError {

  constructor(msg) {
    super(msg || 'Captcha Error');
    this.code = 'P0004';
    this.status = 400;
  }

}

class UsedError extends AbstractError {

  constructor(msg) {
    super(msg || 'Is being used Error');
    this.code = 'P0005';
    this.status = 400;
  }

}

/**
 * exports
 */
module.exports = {
  ParamError,
  PasswordError,
  TypeError,
  EmptyError,
  CaptchaError,
  UsedError
};
