/**
 * @file 系统异常类。
 * @module light.core.error.system
 * @author qiou_kay@163.com
 * @version 1.0.0
 */


'use strict';

const { AbstractError } = require('./common');

/**
 * @desc 系统异常类
 * @param {String} msg 错误信息
 */
class SystemError extends AbstractError {

  constructor(msg) {
    super(msg | 'System Error');
    this.code = 'S0000';
  }

}

/**
 * @desc 配置文件设置错误
 * @param {String} msg 错误信息
 */
class ConfigError extends SystemError {

  constructor(msg) {
    super(msg | 'Config Error');
    this.code = 'S0001';
  }

}

/**
 * @desc 没有权限异常类
 * @param {String} msg 错误信息
 */
class NoPermissionError extends SystemError {

  constructor(msg) {
    super(msg | 'No Permission Error');
    this.code = 'S0002';
  }

}

/**
 * 数据冲突
 * @param {String} msg 错误信息
 */
class ConflictError extends SystemError {

  constructor(msg) {
    super(msg | 'Conflict Error');
    this.code = 'S0003';
  }

}

/**
 * @desc 转换异常类
 * @param {String} msg 错误信息
 */
class ParseError extends SystemError {

  constructor(msg) {
    super(msg | 'Parse Error');
    this.code = 'S0004';
  }

}


/**
 * @desc 文件没有找到异常类
 * @param {String} msg 错误信息
 */
class FileNotFoundError extends SystemError {

  constructor(msg) {
    super(msg | 'File Not Found Error');
    this.code = 'S0005';
  }

}

/**
 * @desc 类没有找到异常类
 * @param {String} msg 错误信息
 */
class ClassNotFoundError extends SystemError {

  constructor(msg) {
    super(msg | 'Class Not Found Error');
    this.code = 'S0006';
  }

}

/**
 * @desc 方法没有找到异常类
 * @param {String} msg 错误信息
 */
class MethodNotFoundError extends SystemError {

  constructor(msg) {
    super(msg | 'Method Not Found Error');
    this.code = 'S0007';
  }

}

/**
 * exports
 */
module.exports = {
  SystemError,
  ConfigError,
  NoPermissionError,
  ConflictError,
  ParseError,
  FileNotFoundError,
  ClassNotFoundError,
  MethodNotFoundError
};
