/**
 * access-control.js
 */

const enforcer         = require('../permission');
const { Unauthorized } = require('../error/http');
const CONST            = require('../constant');
const debug            = require('debug')('light:middleware:access-control');

class AccessControl {

  static use(app) {

    app.use(async (ctx, next) => {

      // 未登陆，不检查权限
      const user = ctx.session[CONST.SESSION.USER];
      if (!user || !user._id) {
        return next();
      }

      const tenant = ctx.session[CONST.SESSION.CODE] || CONST.DEFAULT_TENANT;
      debug('check enforce', tenant, user._id, ctx.path);

      // 校验请求的API的权限
      if (ctx.path.startsWith('/api/')) {
        const allow = await enforcer.enforce(tenant, user._id, ctx.path, 'api', 'ALLOW');
        if (allow) {
          return await next();
        }
      }

      // 校验请求的URL的权限
      const allow = await enforcer.enforce(tenant, user._id, ctx.path, 'route', 'ALLOW');
      if (allow) {
        return await next();
      }

      debug('permission denied', user._id, ctx.path);
      ctx.throw(new Unauthorized('PERMISSION DENIED'));

    });

  }

}

module.exports = AccessControl;
