/**
 * 记录操作日志
 */

const morgan = require('morgan');
const dayjs  = require('dayjs');

class AccessLog {

  static use(app) {

    app.use((ctx, next) => {

      morgan.token('date', () => dayjs().format('YYYY-MM-DDTHH:mm:ss.SSS'));
      morgan.token('uid', () => {
        if (ctx.session && ctx.session.user && ctx.session.user._id) {
          return ctx.session.user._id;
        }
        return '-';
      });

      const options = { skip: req => req.url.match(/\/static.*/i) || req.url.match(/\/health-check.*/i) };
      const format = ['[:date]', '[I]', '[:method]', ':url', ':status',
        ':res[content-length]', ':uid', ':response-time', ':remote-addr'].join(' ');

      return new Promise((resolve, reject) => {
        morgan(format, options)(ctx.req, ctx.res, err => {
          err ? reject('Write operation log failed') : resolve();
        });
      }).then(next);

    });
  }

}

module.exports = AccessLog;
