/**
 * 基于cookie的认证，需要session支持
 */

'use strict';

const debug            = require('debug')('light:middleware:auth-cookie');
const config           = require('../configuration');
const CONST            = require('../constant');
const helper           = require('../helper');
const log              = require('../log');
const { Unauthorized } = require('../error/http');

class AuthCookie {

  static use(app) {
    app.use(async (ctx, next) => {

      // JWT token 认证
      if (ctx[CONST.SYMBOL.JWT]) {
        debug('auth by jwt');
        return await next();
      }

      // OAuth 认证
      if (ctx[CONST.SYMBOL.OAUTH]) {
        debug('auth by oauth');
        return await next();
      }

      // 确认Session里是否有用户情报
      // TODO: 这里固定使用了user, user._id 进行校验，应该时App自己可以配置的
      if (ctx.session && ctx.session.user && ctx.session.user._id) {
        debug('auth by cookie');
        return await next();
      }

      const { auth: ignore = [] } = config.ignore || {};

      // 确认URL是否与不需要认证的路径匹配（配置文件中定义）
      for (const item of ignore) {
        if (ctx.path.match(new RegExp(item, 'i'))) {
          debug('ignore auth check', ctx.path);
          return await next();
        }
      }

      const { home } = config.app || {};

      // 如果是浏览器访问，出现未认证错误，则跳转到主页
      debug('is browser', helper.isBrowser(ctx));
      if (helper.isBrowser(ctx) && home) {

        // 防止循环跳转
        if (ctx.path !== home) {
          log.debug('NOT LOGGED IN');
          debug('redirect to', home);
          return ctx.redirect(home);
        }
      }

      debug('not logged in');
      throw new Unauthorized('NOT LOGGED IN');
    });
  }

}

module.exports = AuthCookie;
