/**
 * 基于jwt的认证，
 * 因为会尝试从body提取内容，所以依赖body-parse中间件
 */

'use strict';

const jwt    = require('jwt-simple');
const log    = require('../log');
const config = require('../configuration');
const CONST  = require('../constant');
const { Unauthorized } = require('../error/http');

class AuthJwt {

  static use(app) {

    app.use(async (ctx, next) => {

      const { tokenSecret = 'light' } = config.app || {};

      // 分别尝试从header，body，query里获取token，如果设置了token尝试解析
      let data = null;
      try {
        if (ctx.get('x-access-token')) {
          data = jwt.decode(ctx.get('x-access-token'), tokenSecret) || {};
        }

        if (!data && ctx.request.body && ctx.request.body.access_token) {
          data = jwt.decode(ctx.request.body.access_token, tokenSecret) || {};
        }

        if (!data && ctx.request.query && ctx.request.query.access_token) {
          data = jwt.decode(ctx.request.query.access_token, tokenSecret) || {};
        }
      } catch (err) {
        log.error('Unable to parse token.', err.message);
      }

      // 保存认证信息
      if (data && data._id) {

        // 确认有效期限
        if (data.expires && data.expires < new Date().getTime()) {
          log.error('Token has expired.');
          throw new Unauthorized('NOT LOGGED IN');
        }

        ctx[CONST.SYMBOL.JWT] = data;
      }

      await next();
    });
  }

}

module.exports = AuthJwt;
