/**
 * 解析请求body
 */

const debug = require('debug')('light:middleware:body-parse');
const body  = require('koa-body');

class BodyParse {

  static use(app) {

    /**
     * parse body
     * 支持 application/x-www-form-urlencoded; text/html; charset=utf-8 形式的 POST 请求
     * 如, 支付宝回调的请求使用这种方式
     */
    app.use(async (ctx, next) => {
      if (/x-www-form-urlencoded/.test(ctx.request.headers['content-type'])) {
        ctx.request.headers['content-type'] = 'application/x-www-form-urlencoded';
      }
      await next();
    });

    // 使用koa-body解析
    debug('limit file size 200M');
    app.use(body({
      multipart    : true,
      parsedMethods: ['POST', 'PUT', 'DELETE'],
      formidable   : {
        maxFileSize: 200 * 1024 * 1024 // 允许上传最大文件限制
      }
    }));

  }

}

module.exports = BodyParse;
