/**
 * 安全检查
 */

const debug  = require('debug')('light:middleware:i18n');
const CONST  = require('../constant');
const i18n   = require('../i18n');
const config = require('../configuration');

class I18n {

  static use(app) {
    app.use(async (ctx, next) => {

      // cookie的语言优先
      let language = ctx.cookies.get(CONST.COOKIE_LANG_KEY);

      // 尝试使用header中的accept-language
      if (!language && ctx.get('accept-language')) {
        const [first] = I18n.accepts(ctx);
        if (first) {
          language = first.lang;
        }
      }

      // 尝试使用用户信息中的lang
      if (!language && ctx.session && ctx.session.user) {
        language = ctx.session.user.lang;
      }

      i18n.lang = language || 'zh';

      const { samesite = [] } = config.ignore || {};
      const client = ctx.get('x-client-type');

      debug('language: %s, client: %s', i18n.lang, client);
      ctx.cookies.set(CONST.COOKIE_LANG_KEY, i18n.lang, {
        sameSite: samesite.includes(client) ? 'none' : 'lax'
      });

      await next();
    });
  }

  /**
   * 参考
   * https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Language
   * @param {*} ctx
   */
  static accepts(ctx) {

    const items = ctx.get('accept-language').split(',');
    if (items <= 0) {
      return [];
    }

    return items
      .map(item => {
        const lang = item.split(/-|;|=/);
        return {
          lang: lang[0].trim(),
          q   : item.includes(';q') ? Number(lang[lang.length - 1]) : 1
        };
      })
      .filter(item => item.lang !== '*')
      .sort((val1, val2) => val2.q - val1.q);

  }

}

module.exports = I18n;
