/**
 * 用于模拟DELETE and PUT方法
 * 如，可以在form里放在<input type="hidden" name="_method" value="put" />来模拟
 *
 * 因为需要查询body，需要body-parse中间件支持
 */

const METHODS = [
  'GET',
  'POST',
  'PUT',
  'HEAD',
  'DELETE',
  'OPTIONS',
  'TRACE',
  'COPY',
  'LOCK',
  'MKCOL',
  'MOVE',
  'PURGE',
  'PROPFIND',
  'PROPPATCH',
  'UNLOCK',
  'REPORT',
  'MKACTIVITY',
  'CHECKOUT',
  'MERGE',
  'M-SEARCH',
  'NOTIFY',
  'SUBSCRIBE',
  'UNSUBSCRIBE',
  'PATCH',
  'SEARCH',
  'CONNECT'
];

class MethodOverride {

  static use(app) {

    const ALLOWED = ['POST'];
    const HTTP_METHOD_OVERRIDE_HEADER = 'x-http-method-override';// Google GData
    const HTTP_METHOD_HEADER          = 'x-http-method';         // Microsoft
    const METHOD_OVERRIDE_HEADER      = 'x-method-override';     // IBM
    const METHOD_NAME                 = '_method';

    app.use(async (ctx, next) => {

      // 确认原始的method是否合法
      const original = (ctx.req.originalMethod || ctx.req.method).toUpperCase();
      if (!ALLOWED.includes(original)) {
        return await next();
      }

      // 查询header, body, query
      const method = ctx.get(HTTP_METHOD_OVERRIDE_HEADER)
        || ctx.get(HTTP_METHOD_HEADER)
        || ctx.get(METHOD_OVERRIDE_HEADER)
        || ctx.request.body[METHOD_NAME]
        || ctx.request.query[METHOD_NAME];

      // 确认override的method合法性
      if (method && METHODS.includes(method.toUpperCase())) {
        ctx.req.method = method.toUpperCase();
      }

      await next();
    });

  }

}

module.exports = MethodOverride;
