/**
 * 静态资源解析
 *
 * 只有指定目录下（默认/static）的文件被解析
 * 支持favicon的路径转换
 */

const send   = require('koa-send');
const config = require('../configuration');

class Static {

  static use(app) {
    app.use(async (ctx, next) => {

      const { static: path = '/static' } = config.app || {};
      const regexp = new RegExp(`^${path}.*`);

      // 修改对根目录 /favicon.ico 的请求
      if (ctx.path === '/favicon.ico') {
        ctx.path = `${path}/favicon.ico`;
      }

      // 在静态资源目录下，才解析
      if (ctx.path.match(regexp)) {
        return await send(ctx, ctx.path);
      }

      await next();
    });
  }

}

module.exports = Static;
