/**
 * 静态资源解析
 */

const config = require('../configuration');

class Timeout {

  static use(app) {

    /**
     * Node创建的HTTP服务器，其对于每个客户端请求的默认连接时长是2分钟（120秒）。
     * 如果在这个时间内，服务器没有发送响应信息（response），客户端的连接就会被重置。
     *
     *  server.setTimeout
     *    给所有用户的请求和响应设置超时
     *
     *  response.setTimeout
     *    服务器处理较长时间
     *    用户长时间接收不到服务器响应，影响用户体验和其它用户请求的进入
     *
     *  request.setTimeout
     *    用户请求耗时（如文件上传）
     *    服务器停留在一个用户请求的接收状态，从而阻碍其它用户请求的进入
     */

    app.use(async (ctx, next) => {

      const { timeout: ignore = [] } = config.ignore || {};

      let special = 0;
      for (const item of ignore) {

        const [pattern, timeout] = item.split(' ');

        // 检查是否是ignore的路径
        if (ctx.path.match(new RegExp(pattern, 'i'))) {

          // 获取特定URL的超时时间
          special = Number(timeout);
          if (isNaN(special) || special <= 0) {
            return await next();
          }
          break;
        }
      }

      // 设定响应超时时间(默认3秒)
      const { timeout } = config.app || {};
      const time = special || Number(timeout) || 3;

      ctx.res.setTimeout(time * 1000, () => {
        ctx.respond = false;
        ctx.res.statusCode = 500;
        ctx.res.end('RPOCESS TIMEOUT');
      });

      await next();
    });

  }

}

module.exports = Timeout;
