/**
 * validator.js
 */

'use strict';

const debug          = require('debug')('light:middleware:validator');
const validator      = require('../validator');
const { BadRequest } = require('../error/http');
const Context        = require('../context');

class Validator {

  static use(app) {
    app.use(async (ctx, next) => {

      debug('request', ctx.path);
      const handler = new Context().bind(ctx);

      // 执行校验
      const result = await validator.check(handler, ctx.path);

      // 无异常
      if (!result) {
        debug('parsed');
        return await next();
      }

      // 校验失败
      const error = new BadRequest('VALIDATION FAILED');
      error.errors = result;
      throw error;
    });
  }

}

module.exports = Validator;
