/**
 * 支持 text/xml, application/xml 格式的内容解析
 * 需要是 POST | PUT | PATCH 请求
 */

const xml2js = require('xml2js');
const raw    = require('raw-body');
const CONST  = require('../constant');

class XmlParse {

  static use(app) {

    app.use(async (ctx, next) => {

      // TODO: 腾讯支付回调 - 没有context-type，但内容是xml，这种情况需要想办法对应
      const isXmlRequest = ctx.is('text/xml', 'xml');
      const isValidMethod = ['POST', 'PUT', 'PATCH'].includes(ctx.method);

      if (isXmlRequest && isValidMethod) {
        ctx.request[CONST.SYMBOL.XML] = await XmlParse.parse(ctx);
      }

      await next();
    });

  }

  static parse(ctx) {

    const xmlOption = { explicitArray: false };
    const rawOption = {
      length  : ctx.request.header['content-length'],
      limit   : '10mb',
      encoding: ctx.request.charset | 'utf-8'
    };

    return new Promise((resolve, reject) => {
      raw(ctx.req, rawOption)
        .then(xml => {
          xml2js.parseString(xml, xmlOption, (err, json) => {
            if (err) {
              return reject(err);
            }
            resolve(json);
          });
        })
        .catch(err => reject(err));
    });
  }

}

module.exports = XmlParse;
