/**
 * store.js
 *
 * 用于缓存导入的数据
 * MongoStore   - 将导入的数据保存到MongoDB的表中，数据量非常大的时候为了避免占用大量内存
 * MemoryStore  - 将导入的数据保存到内存中
 */

'use strict';

const Model = require('../../db/mongo/model');

class MongoStore {

  constructor(domain, code, table, options) {
    this.store = new Model(domain, code, table, options);
    this.code = this.store.code;
    this.domain = domain;
    this.table = table;
  }

  async add(data) {
    return this.store.add(data);
  }

  async list() {
    return this.store.list();
  }

  async cursor() {
    return this.store.getCursor();
  }

  async drop() {
    return this.store.dropCollection();
  }

}

class MemoryStore {

  constructor() {
    this.code = '';
    this.store = [];
    this.index = 0;
    this.storeCursor = {
      next: async () => {
        this.index = this.index + 1;
        if (this.index - 1 < this.store.length) {
          return this.store[this.index - 1];
        }
      }
    };
  }

  async add(data) {
    this.store.push(data);
  }

  async list() {
    return this.store;
  }

  async cursor() {
    return this.storeCursor;
  }

  async drop() {
    this.store = [];
  }

}

module.exports = { MemoryStore, MongoStore };
