/**
 * service.js
 */

const debug     = require('debug')('light:service');
const path      = require('path');
const developer = require('./lib/developer');
const log       = require('./lib/log');
const CONST     = require('./lib/constant');
const helper    = require('./lib/helper');
const mq        = require('./lib/mq');

class Service {

  static init() {

    debug('init service app');
    developer.loadServerConfig();

    const option = {
      exchange: `${CONST.MQ.PREFIX}.${process.env.APPNAME}`,
      queue   : `${CONST.MQ.PREFIX}.${process.env.APPNAME}.${process.env.SERVER_QUEUE || 'queue'}`
    };

    debug('option', option);
    mq.subscribe(option, async (err, data) => {

      debug('error', err, 'data', data);
      if (err) {
        log.error(err);
        if (data) {
          data.channel.nack(data.reply);
        }
        return;
      }

      const root = helper.isUnitTest() ? 'test' : '';
      const ctrl = helper.resolve(path.join(
        root,
        CONST.PATH_CONTROLLER,
        process.env.SERVER_ENTERPOINT || 'service'
      ));

      try {
        await ctrl.listener.call(this, {
          message: data.message,
          ack    : () => data.channel.ack(data.reply),
          nack   : () => data.channel.nack(data.reply)
        });
      } catch (err) {
        log.error(err);
      }

    });

  }

}

if (!module.parent) Service.init();
module.exports = Service;
