/**
 * app.test.js
 */

const supertest = require('supertest');
const App = require('../app');
const pool = require('../lib/db/mongo/pool');

beforeAll(async () => {

  process.env.APPLOCAL = true;

  await App.initServer();
  this.request = supertest.agent(App.server);
});

afterAll(async () => {
  App.server.close();
  await pool.destroy();
  delete process.env.APPLOCAL;
});

describe('App', () => {

  it('health check', async () => {
    const res = await this.request.get('/health-check');
    expect(res.status).toBe(200);
    expect(res.body).toEqual({});
  });

  it('login', async () => {
    const res1 = await this.request.get('/api/account/login');
    expect(res1.body.data._id).toBeTruthy();

    const res2 = await this.request.get('/api/user/list');
    expect(res2.body.data.totalItems).toBeGreaterThan(0);
  });

});
