/**
 * pull.test.js
 */

'use strict';

const fs = require('fs');
const file = require('../../lib/model/file');

describe('Push', () => {

  beforeAll(() => {
    process.env.APPLOCAL = true;
  });

  afterAll(() => {
    delete process.env.APPLOCAL;
  });

  it('pull', async () => {

    jest.mock('../../lib/model/datarider', () => ({
      init: jest.fn,
      code: {
        list: () => ({
          items: [
            { // 创建文件
              name  : '/src.txt',
              type  : 'code',
              md5   : '8de92ce2033cf3ca03fa8cc63e7a703f',
              lang  : 'node',
              source: 'val1'
            },
            { // MD5相同，不更新
              name  : '/src.txt',
              type  : 'code',
              md5   : '8de92ce2033cf3ca03fa8cc63e7a703f',
              lang  : 'node',
              source: 'val2'
            },
            { // MD5不同，更新内容
              name  : '/src.txt',
              type  : 'code',
              md5   : '38ceaa3b09c5a07d329888ba1ccde9ad',
              lang  : 'node',
              source: 'val3'
            }, // 二进制文件
            {
              name  : '/binary.txt',
              type  : 'binary',
              md5   : 'md5',
              lang  : 'node',
              source: 'fileid'
            }
          ]
        })
      }
    }));

    const Pull = require('../../bin/pull')
      , spy = jest.spyOn(file, 'stream')
        .mockImplementation(async () => ({
          pipe: jest.fn(),
          on  : (event, fn) => fn()
        }));

    await new Pull().start();

    const path = `${process.cwd()}/tmp/data/${process.env.APPNAME}/src.txt`
      , src = fs.readFileSync(path, 'utf8');

    expect(src).toBe('val3');
    expect(spy.mock.calls[0][0].params.id).toBe('fileid');
  });

});
