/**
 * configuration.test.js
 */

'use strict';

const config = require('../../lib/configuration');

describe('Configuration', () => {

  afterAll(() => {
    delete process.env.APPENV;
  });

  it('load string value', () => {
    config.load([
      {
        type     : 'k1',
        key      : 'k2',
        value    : '/static',
        valueType: 'string'
      }
    ]);
    expect(config.k1.k2).toBe('/static');
  });

  it('load number value', () => {
    config.load([
      {
        type     : 'k1',
        key      : 'k1',
        value    : '168',
        valueType: 'number'
      },
      {
        type     : 'k1',
        key      : 'k2',
        value    : 'bad number',
        valueType: 'number'
      },
      {
        type     : 'k1',
        key      : 'k3',
        value    : 10,
        valueType: 'number'
      }
    ]);
    expect(config.k1.k1).toBe(168);
    expect(config.k1.k2).toBe('bad number');
    expect(config.k1.k3).toBe(10);
  });

  it('load boolean value', () => {
    config.load([
      {
        type     : 'k1',
        key      : 'k2.k1',
        value    : 0,
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k2',
        value    : 1,
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k3',
        value    : 'false',
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k4',
        value    : '0',
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k5',
        value    : '',
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k6',
        value    : 'true',
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k7',
        value    : '1',
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k8',
        value    : false,
        valueType: 'boolean'
      },
      {
        type     : 'k1',
        key      : 'k2.k9',
        value    : true,
        valueType: 'boolean'
      }
    ]);
    expect(config.k1.k2.k1).toBe(false);
    expect(config.k1.k2.k2).toBe(true);
    expect(config.k1.k2.k3).toBe(false);
    expect(config.k1.k2.k4).toBe(false);
    expect(config.k1.k2.k5).toBe(false);
    expect(config.k1.k2.k6).toBe(true);
    expect(config.k1.k2.k7).toBe(true);
    expect(config.k1.k2.k8).toBe(false);
    expect(config.k1.k2.k9).toBe(true);
  });

  it('load array value', () => {
    config.load([
      {
        type     : 'k1',
        key      : 'k3',
        value    : [1, 2],
        valueType: 'array'
      }
    ]);
    expect(config.k1.k3).toEqual([1, 2]);
  });

  it('load product value', () => {
    process.env.APPENV = 'default';
    config.load([
      {
        type     : 'k1',
        key      : 'k2',
        value    : { default: { a: 10, b: 30 } },
        valueType: 'object'
      }
    ]);
    expect(config.k1.k2).toEqual({ a: 10, b: 30 });
  });

});
