/**
 * type.test.js
 */

'use strict';

const type         = require('../../../../lib/db/mongo/type');
const dayjs        = require('dayjs');
const { ObjectId } = require('mongodb');
const objectid     = new ObjectId('5e70e7b55718d603ff4efe5b');

describe('Type', () => {

  it('data parse undefined', () => {
    const arr = type.dataParse(undefined, { type: 'Array' });
    const obj = type.dataParse(undefined, { type: 'Object' });
    const bol = type.dataParse(undefined, { type: 'Boolean' });
    const num = type.dataParse(undefined, { type: 'Number' });
    const dat = type.dataParse(undefined, { type: 'Date' });
    const oid = type.dataParse(undefined, { type: 'ObjectId' });
    const str = type.dataParse(undefined, { type: 'String' });
    expect(arr).toEqual([]);
    expect(obj).toEqual({});
    expect(bol).toBe(false);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('');
  });

  it('data parse null', () => {
    const arr = type.dataParse(null, { type: 'Array' });
    const obj = type.dataParse(null, { type: 'Object' });
    const bol = type.dataParse(null, { type: 'Boolean' });
    const num = type.dataParse(null, { type: 'Number' });
    const dat = type.dataParse(null, { type: 'Date' });
    const oid = type.dataParse(null, { type: 'ObjectId' });
    const str = type.dataParse(null, { type: 'String' });

    expect(arr).toEqual([]);
    expect(obj).toEqual({});
    expect(bol).toEqual(null);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('');
  });

  it('data parse infinity', () => {
    const arr = type.dataParse(Infinity, { type: 'Array' });
    const obj = type.dataParse(Infinity, { type: 'Object' });
    const bol = type.dataParse(Infinity, { type: 'Boolean' });
    const num = type.dataParse(Infinity, { type: 'Number' });
    const dat = type.dataParse(Infinity, { type: 'Date' });
    const oid = type.dataParse(Infinity, { type: 'ObjectId' });
    const str = type.dataParse(Infinity, { type: 'String' });
    expect(arr).toEqual([]);
    expect(obj).toEqual({});
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('');
  });

  it('data parse nan', () => {
    const arr = type.dataParse(NaN, { type: 'Array' });
    const obj = type.dataParse(NaN, { type: 'Object' });
    const bol = type.dataParse(NaN, { type: 'Boolean' });
    const num = type.dataParse(NaN, { type: 'Number' });
    const dat = type.dataParse(NaN, { type: 'Date' });
    const oid = type.dataParse(NaN, { type: 'ObjectId' });
    const str = type.dataParse(NaN, { type: 'String' });
    expect(arr).toEqual([]);
    expect(obj).toEqual({});
    expect(bol).toEqual(false);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('');
  });

  it('data parse empty string', () => {
    const arr = type.dataParse('', { type: 'Array' });
    const obj = type.dataParse('', { type: 'Object' });
    const bol = type.dataParse('', { type: 'Boolean' });
    const num = type.dataParse('', { type: 'Number' });
    const dat = type.dataParse('', { type: 'Date' });
    const oid = type.dataParse('', { type: 'ObjectId' });
    const str = type.dataParse('', { type: 'String' });
    expect(arr).toEqual([]);
    expect(obj).toEqual({});
    expect(bol).toEqual(false);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('');
  });

  it('data parse null string', () => {
    const arr = type.dataParse('null', { type: 'Array' });
    const obj = type.dataParse('null', { type: 'Object' });
    const bol = type.dataParse('null', { type: 'Boolean' });
    const num = type.dataParse('null', { type: 'Number' });
    const dat = type.dataParse('null', { type: 'Date' });
    const oid = type.dataParse('null', { type: 'ObjectId' });
    const str = type.dataParse('null', { type: 'String' });
    expect(arr).toEqual([]);
    expect(obj).toEqual({});
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('null');
  });

  it('data parse space string', () => {
    const arr = type.dataParse(' ', { type: 'Array' });
    const obj = type.dataParse(' ', { type: 'Object' });
    const bol = type.dataParse(' ', { type: 'Boolean' });
    const num = type.dataParse(' ', { type: 'Number' });
    const dat = type.dataParse(' ', { type: 'Date' });
    const oid = type.dataParse(' ', { type: 'ObjectId' });
    const str = type.dataParse(' ', { type: 'String' });
    expect(arr).toEqual([]);
    expect(obj).toEqual({});
    expect(bol).toEqual(false);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe(' ');
  });

  it('data parse zero', () => {
    const arr = type.dataParse(0, { type: 'Array' });
    const obj = type.dataParse(0, { type: 'Object' });
    const bol = type.dataParse(0, { type: 'Boolean' });
    const num = type.dataParse(0, { type: 'Number' });
    const dat = type.dataParse(0, { type: 'Date' });
    const oid = type.dataParse(0, { type: 'ObjectId' });
    const str = type.dataParse(0, { type: 'String' });
    expect(arr).toEqual([0]);
    expect(obj).toEqual(0);
    expect(bol).toEqual(false);
    expect(num).toBe(0);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('0');
  });

  it('data parse one', () => {
    const arr = type.dataParse(1, { type: 'Array' });
    const obj = type.dataParse(1, { type: 'Object' });
    const bol = type.dataParse(1, { type: 'Boolean' });
    const num = type.dataParse(1, { type: 'Number' });
    const dat = type.dataParse(1, { type: 'Date' });
    const oid = type.dataParse(1, { type: 'ObjectId' });
    const str = type.dataParse(1, { type: 'String' });
    expect(arr).toEqual([1]);
    expect(obj).toEqual(1);
    expect(bol).toEqual(true);
    expect(num).toBe(1);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('1');
  });

  it('data parse boolean object', () => {
    // eslint-disable-next-line no-new-wrappers
    const val = new Boolean(2);
    const arr = type.dataParse(val, { type: 'Array' });
    const obj = type.dataParse(val, { type: 'Object' });
    const bol = type.dataParse(val, { type: 'Boolean' });
    const num = type.dataParse(val, { type: 'Number' });
    const dat = type.dataParse(val, { type: 'Date' });
    const oid = type.dataParse(val, { type: 'ObjectId' });
    const str = type.dataParse(val, { type: 'String' });
    expect(arr).toEqual([val]);
    expect(obj).toBe(val);
    expect(bol).toBe(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('true');
  });

  it('data parse string number', () => {
    const arr = type.dataParse('1', { type: 'Array' });
    const obj = type.dataParse('1', { type: 'Object' });
    const bol = type.dataParse('1', { type: 'Boolean' });
    const num = type.dataParse('1', { type: 'Number' });
    const dat = type.dataParse('1', { type: 'Date' });
    const oid = type.dataParse('1', { type: 'ObjectId' });
    const str = type.dataParse('1', { type: 'String' });
    expect(arr).toEqual([1]);
    expect(obj).toEqual('1');
    expect(bol).toEqual(true);
    expect(num).toBe(1);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('1');
  });

  it('data parse string number zero', () => {
    const arr = type.dataParse('0', { type: 'Array' });
    const obj = type.dataParse('0', { type: 'Object' });
    const bol = type.dataParse('0', { type: 'Boolean' });
    const num = type.dataParse('0', { type: 'Number' });
    const dat = type.dataParse('0', { type: 'Date' });
    const oid = type.dataParse('0', { type: 'ObjectId' });
    const str = type.dataParse('0', { type: 'String' });
    expect(arr).toEqual([0]);
    expect(obj).toEqual('0');
    expect(bol).toEqual(false);
    expect(num).toBe(0);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('0');
  });

  it('data parse boolean true', () => {
    const arr = type.dataParse(true, { type: 'Array' });
    const obj = type.dataParse(true, { type: 'Object' });
    const bol = type.dataParse(true, { type: 'Boolean' });
    const num = type.dataParse(true, { type: 'Number' });
    const dat = type.dataParse(true, { type: 'Date' });
    const oid = type.dataParse(true, { type: 'ObjectId' });
    const str = type.dataParse(true, { type: 'String' });
    expect(arr).toEqual([true]);
    expect(obj).toBe(true);
    expect(bol).toBe(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('true');
  });

  it('data parse boolean false', () => {
    const arr = type.dataParse(false, { type: 'Array' });
    const obj = type.dataParse(false, { type: 'Object' });
    const bol = type.dataParse(false, { type: 'Boolean' });
    const num = type.dataParse(false, { type: 'Number' });
    const dat = type.dataParse(false, { type: 'Date' });
    const oid = type.dataParse(false, { type: 'ObjectId' });
    const str = type.dataParse(false, { type: 'String' });
    expect(arr).toEqual([false]);
    expect(obj).toBe(false);
    expect(bol).toBe(false);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('false');
  });

  it('data parse boolean object', () => {
    // eslint-disable-next-line no-new-wrappers
    const val = new Boolean(true);
    const arr = type.dataParse(val, { type: 'Array' });
    const obj = type.dataParse(val, { type: 'Object' });
    const bol = type.dataParse(val, { type: 'Boolean' });
    const num = type.dataParse(val, { type: 'Number' });
    const dat = type.dataParse(val, { type: 'Date' });
    const oid = type.dataParse(val, { type: 'ObjectId' });
    const str = type.dataParse(val, { type: 'String' });
    expect(arr).toEqual([val]);
    expect(obj).toEqual(val);
    expect(bol).toBe(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('true');
  });

  it('data parse string', () => {
    const arr = type.dataParse('a', { type: 'Array' });
    const obj = type.dataParse('a', { type: 'Object' });
    const bol = type.dataParse('a', { type: 'Boolean' });
    const num = type.dataParse('a', { type: 'Number' });
    const dat = type.dataParse('a', { type: 'Date' });
    const oid = type.dataParse('a', { type: 'ObjectId' });
    const str = type.dataParse('a', { type: 'String' });
    expect(arr).toEqual(['a']);
    expect(obj).toEqual('a');
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('a');
  });

  it('data parse string object', () => {
    // eslint-disable-next-line no-new-wrappers
    const val = new String('a');
    const arr = type.dataParse(val, { type: 'Array' });
    const obj = type.dataParse(val, { type: 'Object' });
    const bol = type.dataParse(val, { type: 'Boolean' });
    const num = type.dataParse(val, { type: 'Number' });
    const dat = type.dataParse(val, { type: 'Date' });
    const oid = type.dataParse(val, { type: 'ObjectId' });
    const str = type.dataParse(val, { type: 'String' });
    expect(arr).toEqual(['a']);
    expect(obj).toBe('a');
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('a');
  });

  it('data parse json array string', () => {
    const arr = type.dataParse('[1]', { type: 'Array' });
    const obj = type.dataParse('[1]', { type: 'Object' });
    const bol = type.dataParse('[1]', { type: 'Boolean' });
    const num = type.dataParse('[1]', { type: 'Number' });
    const dat = type.dataParse('[1]', { type: 'Date' });
    const oid = type.dataParse('[1]', { type: 'ObjectId' });
    const str = type.dataParse('[1]', { type: 'String' });
    expect(arr).toEqual([1]);
    expect(obj).toEqual([1]);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('[1]');
  });

  it('data parse json object string', () => {
    const arr = type.dataParse('{"v": 1}', { type: 'Array' });
    const obj = type.dataParse('{"v":1}', { type: 'Object' });
    const bol = type.dataParse('{"v":1}', { type: 'Boolean' });
    const num = type.dataParse('{"v":1}', { type: 'Number' });
    const dat = type.dataParse('{"v":1}', { type: 'Date' });
    const oid = type.dataParse('{"v":1}', { type: 'ObjectId' });
    const str = type.dataParse('{"v":1}', { type: 'String' });
    expect(arr).toEqual([{ v: 1 }]);
    expect(obj).toEqual({ v: 1 });
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('{"v":1}');
  });

  it('data parse json', () => {
    const arr = type.dataParse({ v: 1 }, { type: 'Array' });
    const obj = type.dataParse({ v: 1 }, { type: 'Object' });
    const bol = type.dataParse({ v: 1 }, { type: 'Boolean' });
    const num = type.dataParse({ v: 1 }, { type: 'Number' });
    const dat = type.dataParse({ v: 1 }, { type: 'Date' });
    const oid = type.dataParse({ v: 1 }, { type: 'ObjectId' });
    const str = type.dataParse({ v: 1 }, { type: 'String' });
    expect(arr).toEqual([{ v: 1 }]);
    expect(obj).toEqual({ v: 1 });
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('{"v":1}');
  });

  it('data parse array', () => {
    const arr = type.dataParse([1], { type: 'Array' });
    const obj = type.dataParse([1], { type: 'Object' });
    const bol = type.dataParse([1], { type: 'Boolean' });
    const num = type.dataParse([1], { type: 'Number' });
    const dat = type.dataParse([1], { type: 'Date' });
    const oid = type.dataParse([1], { type: 'ObjectId' });
    const str = type.dataParse([1], { type: 'String' });
    expect(arr).toEqual([1]);
    expect(obj).toEqual([1]);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('[1]');
  });

  it('data parse empty json', () => {
    const arr = type.dataParse({}, { type: 'Array' });
    const obj = type.dataParse({}, { type: 'Object' });
    const bol = type.dataParse({}, { type: 'Boolean' });
    const num = type.dataParse({}, { type: 'Number' });
    const dat = type.dataParse({}, { type: 'Date' });
    const oid = type.dataParse({}, { type: 'ObjectId' });
    const str = type.dataParse({}, { type: 'String' });
    expect(arr).toEqual([{}]);
    expect(obj).toEqual({});
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('{}');
  });

  it('data parse objectid', () => {
    const arr = type.dataParse(objectid, { type: 'Array' });
    const obj = type.dataParse(objectid, { type: 'Object' });
    const bol = type.dataParse(objectid, { type: 'Boolean' });
    const num = type.dataParse(objectid, { type: 'Number' });
    const dat = type.dataParse(objectid, { type: 'Date' });
    const oid = type.dataParse(objectid, { type: 'ObjectId' });
    const str = type.dataParse(objectid, { type: 'String' });
    expect(arr).toEqual([objectid]);
    expect(obj).toEqual(objectid);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(objectid);
    expect(str).toBe('5e70e7b55718d603ff4efe5b');
  });

  it('data parse date', () => {
    const day = new Date('2020-02-01T16:00:00.000Z');
    const arr = type.dataParse(day, { type: 'Array' });
    const obj = type.dataParse(day, { type: 'Object' });
    const bol = type.dataParse(day, { type: 'Boolean' });
    const num = type.dataParse(day, { type: 'Number' });
    const dat = type.dataParse(day, { type: 'Date' });
    const oid = type.dataParse(day, { type: 'ObjectId' });
    const str = type.dataParse(day, { type: 'String' });
    expect(arr).toEqual([day]);
    expect(obj).toEqual(day);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(day);
    expect(oid).toBe(null);
    expect(str).toBe('2020-02-01T16:00:00.000Z');
  });

  it('data parse date string', () => {
    const day = '2020-02-01T16:00:00.000Z';
    const arr = type.dataParse(day, { type: 'Array' });
    const obj = type.dataParse(day, { type: 'Object' });
    const bol = type.dataParse(day, { type: 'Boolean' });
    const num = type.dataParse(day, { type: 'Number' });
    const dat = type.dataParse(day, { type: 'Date' });
    const oid = type.dataParse(day, { type: 'ObjectId' });
    const str = type.dataParse(day, { type: 'String' });
    expect(arr).toEqual([day]);
    expect(obj).toEqual(day);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toEqual(new Date(day));
    expect(oid).toBe(null);
    expect(str).toBe('2020-02-01T16:00:00.000Z');
  });

  it('data parse dayjs', () => {
    const day = dayjs('2020-02-01T16:00:00.000Z');
    const arr = type.dataParse(day, { type: 'Array' });
    const obj = type.dataParse(day, { type: 'Object' });
    const bol = type.dataParse(day, { type: 'Boolean' });
    const num = type.dataParse(day, { type: 'Number' });
    const dat = type.dataParse(day, { type: 'Date' });
    const oid = type.dataParse(day, { type: 'ObjectId' });
    const str = type.dataParse(day, { type: 'String' });
    expect(arr).toEqual([day]);
    expect(obj).toEqual(day);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(day);
    expect(oid).toBe(null);
    expect(str).toBe('2020-02-01T16:00:00.000Z');
  });

  it('data parse regex', () => {
    const val = /^$/;
    const arr = type.dataParse(val, { type: 'Array' });
    const obj = type.dataParse(val, { type: 'Object' });
    const bol = type.dataParse(val, { type: 'Boolean' });
    const num = type.dataParse(val, { type: 'Number' });
    const dat = type.dataParse(val, { type: 'Date' });
    const oid = type.dataParse(val, { type: 'ObjectId' });
    const str = type.dataParse(val, { type: 'String' });
    expect(arr).toEqual([val]);
    expect(obj).toEqual(val);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('/^$/');
  });

  it('data parse regex object', () => {
    const val = new RegExp('^$');
    const arr = type.dataParse(val, { type: 'Array' });
    const obj = type.dataParse(val, { type: 'Object' });
    const bol = type.dataParse(val, { type: 'Boolean' });
    const num = type.dataParse(val, { type: 'Number' });
    const dat = type.dataParse(val, { type: 'Date' });
    const oid = type.dataParse(val, { type: 'ObjectId' });
    const str = type.dataParse(val, { type: 'String' });
    expect(arr).toEqual([val]);
    expect(obj).toEqual(val);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe('/^$/');
  });

  it('query parse regex', () => {
    const val = /^$/;
    const arr = type.queryParse(val, { type: 'Array' });
    const obj = type.queryParse(val, { type: 'Object' });
    const bol = type.queryParse(val, { type: 'Boolean' });
    const num = type.queryParse(val, { type: 'Number' });
    const dat = type.queryParse(val, { type: 'Date' });
    const oid = type.queryParse(val, { type: 'ObjectId' });
    const str = type.queryParse(val, { type: 'String' });
    expect(arr).toEqual([val]);
    expect(obj).toEqual(val);
    expect(bol).toEqual(true);
    expect(num).toBe(null);
    expect(dat).toBe(null);
    expect(oid).toBe(null);
    expect(str).toBe(val);
  });

  it('unsupport type', () => {
    const dummy = type.queryParse(10, { type: 'Function' });
    expect(dummy).toEqual(10);
  });

  it('is blaank', () => {
    expect(type.isBlank()).toBe(true);
    expect(type.isBlank(undefined)).toBe(true);
    expect(type.isBlank(null)).toBe(true);
    expect(type.isBlank(NaN)).toBe(true);
    expect(type.isBlank(Infinity)).toBe(true);
    expect(type.isBlank('\n')).toBe(true);
    expect(type.isBlank('\r')).toBe(true);
    expect(type.isBlank('\t')).toBe(true);
    expect(type.isBlank('')).toBe(true);
    expect(type.isBlank(' ')).toBe(true);
    expect(type.isBlank(0)).toBe(false);
    expect(type.isBlank('0')).toBe(false);
  });

  it('is json', () => {

    class Clazz {
    }

    expect(type.isJson()).toBe(false);
    expect(type.isJson(undefined)).toBe(false);
    expect(type.isJson(null)).toBe(false);
    expect(type.isJson(NaN)).toBe(false);
    expect(type.isJson(Infinity)).toBe(false);
    expect(type.isJson('')).toBe(false);
    expect(type.isJson(0)).toBe(false);
    expect(type.isJson(true)).toBe(false);
    expect(type.isJson(Clazz)).toBe(false);
    expect(type.isJson(new Clazz())).toBe(false);
    expect(type.isJson(/^$/)).toBe(false);
    expect(type.isJson([])).toBe(false);
    expect(type.isJson(new Date())).toBe(false);
    expect(type.isJson(dayjs())).toBe(false);
    expect(type.isJson(new Map())).toBe(false);
    expect(type.isJson(new Set())).toBe(false);
    expect(type.isJson(Symbol.for(''))).toBe(false);
    expect(type.isJson({})).toBe(true);

    // eslint-disable-next-line no-new-object
    expect(type.isJson(new Object())).toBe(true);
  });

});
