/**
 * helper.test.js
 */

'use strict';

const qs = require('qs');
const aksk = require('../../../lib/helper/aksk');

describe('Helper', () => {

  it('nonce', () => {
    expect(aksk.nonce()).toHaveLength(13);
    expect(aksk.nonce(0)).toHaveLength(13);
    expect(aksk.nonce(1)).toHaveLength(14);
    expect(aksk.nonce(10)).toHaveLength(23);
  });

  it('escape', () => {
    expect(aksk.escape('')).toBe('');
    // escape
    expect(aksk.escape('!')).toBe('%21');
    expect(aksk.escape('\'')).toBe('%27');
    expect(aksk.escape('(')).toBe('%28');
    expect(aksk.escape(')')).toBe('%29');
    expect(aksk.escape('*')).toBe('%2A');
    // encode
    expect(aksk.escape('$')).toBe('%24');
    expect(aksk.escape('&')).toBe('%26');
  });

  it('querystring', () => {
    expect(aksk.querystring({})).toBe('');
    expect(aksk.querystring({ a: 1 })).toBe('a%3D1');
    expect(aksk.querystring({ a: 1, b: 2 })).toBe('a%3D1%26b%3D2');
    expect(aksk.querystring({ b: 2, a: 1 })).toBe('a%3D1%26b%3D2');
    expect(aksk.querystring({ '(': ')' })).toBe('%2528%3D%2529');

    expect(aksk.querystring({ 我: '你', 你: '我' })).toBe('%25E4%25BD%25A0%3D%25E6%2588%2591%26%25E6%2588%2591%3D%25E4%25BD%25A0');
    expect(aksk.querystring({ 你: '我', 我: '你' })).toBe('%25E4%25BD%25A0%3D%25E6%2588%2591%26%25E6%2588%2591%3D%25E4%25BD%25A0');
  });

  it('timestamp', () => {
    expect(aksk.timestamp()).toHaveLength(20);
    expect(aksk.timestamp()).toContain('T');
    expect(aksk.timestamp()).toContain('Z');
  });

  it('sign', () => {
    expect(aksk.sign('key', 'POST', {})).toBe('7qxk7AMrjmidrnBxLCzgEcwRUMA%3D');
    expect(aksk.sign('key', 'GET', { a: 1 })).toBe('8AMppRLhjRNZ%2BRYr%2Bi49hwak8P4%3D');
    expect(aksk.sign('key', 'GET', { a: 1, b: 2 })).toBe('aDgrXKSYLHGul2mvnUEfLjcCuHs%3D');
    expect(aksk.sign('key', 'GET', { b: 2, a: 1 })).toBe('aDgrXKSYLHGul2mvnUEfLjcCuHs%3D');
    expect(aksk.sign('key', 'GET', { 你: '我' })).toBe('pvZ29oal%2FqKQJ5OVZY6hPHDKEJM%3D');
  });


  it('encode', () => {

    const resut = aksk.encode({
      accesskey: '12345',
      secretkey: '67890',
      address  : 'https://admin.zf.link',
      key      : 'value'
    });

    const [address, query] = resut.split('?');
    const data = qs.parse(query);

    expect(address).toBe('https://admin.zf.link');
    expect(data.method).toBe('HMAC-SHA1');
    expect(data.nonce).toHaveLength(17);
    expect(data.timestamp).toHaveLength(20);
    expect(data.accesskey).toBe('12345');
    expect(data.signature).toHaveLength(28);
    expect(data.key).toBe('value');
    expect(data.secretkey).toBeUndefined();
  });

  it('verify', () => {
    expect(aksk.verify('67890', {
      method   : 'HMAC-SHA1',
      nonce    : '16146694408412874',
      timestamp: '2021-03-02T07:17:20Z',
      accesskey: '12345',
      key      : 'value',
      signature: 'T4hSoEipc7u03ehuf2N/+vG4Hh4='
    })).toBe(true);

  });

});

