/**
 * middleware/access-log.test.js
 */

'use strict';

const util = require('./util');

describe('AccessLog', () => {

  beforeAll(() => {
    util.setupKoa(['access-log', 'helloword']);
  });

  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  // 确认访问日志正确
  it('use access-log', async () => {
    const spy = jest.spyOn(process.stdout, 'write');

    await util.request.get('/test');

    const [date, kind, method, url, status, size, uid, time, addr] = spy.mock.calls[0][0].split(' ');
    expect(date).toHaveLength(25);
    expect(kind).toBe('[I]');
    expect(method).toBe('[GET]');
    expect(url).toBe('/test');
    expect(status).toBe('200');
    expect(size).toBe('11');
    expect(uid).toBe('-');
    expect(time).toMatch(/\d.\d*/);
    expect(addr).toMatch('::ffff:127.0.0.1');
  });

  // 确认访问日志正确（指定了uid时）
  it('use access-log (has user id)', async () => {
    util.app.context.session = { user: { _id: '001' } };
    const spy = jest.spyOn(process.stdout, 'write');
    await util.request.get('/test');
    expect(spy.mock.calls[0][0].split(' ')[6]).toBe('001');
  });

});
