/**
 * middleware/auth-cookie.test.js
 */

'use strict';

const util    = require('./util');
const user    = require('../../controllers/user');
const config  = require('../../../lib/configuration');
const setting = require('../../../lib/setting');

describe('AuthCookiet', () => {

  beforeAll(() => {
    util.mockSessionStore();
    setting.board = [
      { api: '/api/account/login', class: 'user', action: 'get' },
      { api: '/api/user/get', class: 'user', action: 'get' },
      { api: '/api/user/update', class: 'user', action: 'update' }
    ];
    util.setupKoa(['session', 'auth-cookie', 'router']);
  });

  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  // 认证失败
  it('unable to authenticate', async () => {

    jest.spyOn(user, 'get');
    const res = await util.request.get('/api/user/get');

    expect(res.body.error.message).toBe('NOT LOGGED IN');
    expect(res.status).toBe(401);
  });

  // 认证失败后跳转
  it('unable to authenticate (redirect)', async () => {

    config.app = { home: '/index.html' };

    jest.spyOn(user, 'update');
    const res = await util.request.post('/api/user/update')
      .set('User-Agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:12.0) Gecko/20100101 Firefox/12.0');

    expect(res.redirect).toBe(true);
    expect(res.header.location).toBe('/index.html');
  });

  // 为后续测试生成认证信息
  it('ignore auth', async () => {

    config.ignore = { auth: ['/api/account/login'] };

    const spy = jest.spyOn(user, 'get')
      .mockImplementation(async ({ ctx }) => ctx.session.user = { _id: 1 });

    await util.request.get('/api/account/login');
    expect(spy).toHaveBeenCalled();
  });

  // 认证成功
  it('can be certified', async () => {

    const spy = jest.spyOn(user, 'update');
    const res = await util.request.get('/api/user/update');

    expect(res.body).toEqual({ apiVersion: '1.0', data: {} });
    expect(spy).toHaveBeenCalled();
  });

});
