/**
 * middleware/auth-jwt.test.js
 */

'use strict';

const jwt     = require('jwt-simple');
const util    = require('./util');
const CONST   = require('../../../lib/constant');
const user    = require('../../controllers/user');
const config  = require('../../../lib/configuration');
const setting = require('../../../lib/setting');

describe('AuthJwt', () => {

  beforeAll(() => {
    setting.board = [{ api: '/api/user/get', class: 'user', action: 'get' }];
    util.setupKoa(['body-parse', 'auth-jwt', 'router']);
  });

  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  // 使用token访问
  it('use token', async () => {

    const payload = {
      _id     : CONST.DEFAULT_USER,
      timezone: 'Asia/Tokyo',
      expires : new Date().getTime() + 500
    };

    config.app = { tokenSecret: 'secret' };

    const token = jwt.encode(payload, config.app.tokenSecret, 'HS256');
    const spy = jest.spyOn(user, 'get');

    await util.request.get('/api/user/get').set('x-access-token', token);
    expect(spy.mock.calls[0][0].uid).toBe(CONST.DEFAULT_USER);
    expect(spy.mock.calls[0][0].timezone).toBe('Asia/Tokyo');

    await util.request.get(`/api/user/get?access_token=${token}`);
    expect(spy.mock.calls[1][0].uid).toBe(CONST.DEFAULT_USER);
    expect(spy.mock.calls[1][0].timezone).toBe('Asia/Tokyo');

    await util.request.post('/api/user/get').send(`access_token=${token}`);
    expect(spy.mock.calls[2][0].uid).toBe(CONST.DEFAULT_USER);
    expect(spy.mock.calls[2][0].timezone).toBe('Asia/Tokyo');
  });

  // token超时
  it('token timeout', async () => {

    const payload = { _id: CONST.DEFAULT_USER, expires: new Date().getTime() };
    const token = jwt.encode(payload, config.app.tokenSecret, 'HS256');
    const spy = jest.spyOn(user, 'get');

    await util.request.get('/api/user/get').set('x-access-token', token);
    expect(spy.mock.calls[0][0].uid).toBeUndefined();
  });

  // 错误的token
  it('token invalid', async () => {
    const spy = jest.spyOn(user, 'get');
    await util.request.get('/api/user/get').set('x-access-token', 'invalid token');
    expect(spy.mock.calls[0][0].uid).toBeUndefined();
  });

});
