/**
 * middleware/body-parse.test.js
 */

'use strict';

const util    = require('./util');
const setting = require('../../../lib/setting');
const user    = require('../../controllers/user');

describe('BodyParse', () => {

  beforeAll(() => {
    setting.board = [{ api: '/api/user/get', class: 'user', action: 'get' }];
    util.setupKoa(['body-parse', 'router']);
  });

  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  // JSON
  it('parse application/json', async () => {
    const spy = jest.spyOn(user, 'get');
    await util.request.post('/api/user/get').send({
      name     : 'light',
      condition: { age: { $gt: 18 } }
    });
    expect(spy.mock.calls[0][0].params).toEqual({ name: 'light', condition: { age: { $gt: 18 } } });
  });

  // 表单
  it('parse application/x-www-form-urlencoded', async () => {
    const spy = jest.spyOn(user, 'get');
    await util.request.post('/api/user/get').send('name=light');
    expect(spy.mock.calls[0][0].params).toEqual({ name: 'light' });
  });

  // 支付宝混合格式
  it('parse application/x-www-form-urlencoded; text/html;', async () => {
    const spy = jest.spyOn(user, 'get');
    await util.request.post('/api/user/get')
      .set('content-type', 'application/x-www-form-urlencoded; text/html; charset=utf-8')
      .send('name=light');
    expect(spy.mock.calls[0][0].params).toEqual({ name: 'light' });
  });

  // 文件
  it('parse file', async () => {
    const spy = jest.spyOn(user, 'get');
    await util.request.post('/api/user/get')
      .field('name', 'light')
      .attach('A', './package.json')
      .attach('B', './README.md')
      .attach('B', './LICENSE');
    expect(spy.mock.calls[0][0].params.name).toBe('light');
    expect(spy.mock.calls[0][0].params.files).toHaveLength(3);
  });

});
