/**
 * middleware/csrf-check.test.js
 */

'use strict';

const util = require('./util');
const config = require('../../../lib/configuration');

describe('CsrfCheck', () => {

  beforeAll(() => {
    util.mockSessionStore();
    util.setupKoa(['body-parse', 'session', 'csrf-check', 'helloword']);
  });

  afterAll(() => util.clear());

  it('get csrf', async () => {
    const { header, text } = await util.request.get('/');
    expect(header.csrftoken).toHaveLength(36);
    expect(header['set-cookie'][0]).toContain(`light.csrf=${header.csrftoken}`);
    expect(text).toBe('Hello World');
    this.csrf = header.csrftoken;
  });

  it('csrf by header', async () => {
    const res = await util.request.post('/index.html').set('csrf-token', this.csrf);
    expect(res.text).toBe('Hello World');
  });

  it('csrf by body', async () => {
    const res = await util.request.post('/index.html').send({ _csrf: this.csrf });
    expect(res.text).toBe('Hello World');
  });

  it('csrf by query', async () => {
    const res = await util.request.post(`/index.html?_csrf=${this.csrf}`);
    expect(res.text).toBe('Hello World');
  });

  it('invalid csrf', async () => {
    const res = await util.request.post('/index.html');
    expect(res.text).toBe('EBADCSRFTOKEN');
  });

  it('ignore csrf', async () => {
    config.ignore = { csrf: ['^/api/user/get', '/api/account/.*'] };
    expect((await util.request.post('/api/user/get?id=light')).text).toBe('Hello World');
    expect((await util.request.post('/api/account/csrf')).text).toBe('Hello World');
    expect((await util.request.post('/api/non-igonre')).text).toBe('EBADCSRFTOKEN');
  });

});
