/**
 * middleware/i18n.test.js
 */

'use strict';

const util    = require('./util');
const i18n    = require('../../../lib/i18n');
const user    = require('../../controllers/user');
const setting = require('../../../lib/setting');

describe('I18n', () => {

  beforeAll(() => {
    setting.board = [{ api: '/api/user/get', class: 'user', action: 'get' }];
    jest.spyOn(user, 'get');
    util.setupKoa(['i18n', 'router'], false);
  });

  afterAll(() => util.clear());

  // 解析accept-language
  it('use accept-language', async () => {
    const res = await util.request.get('/api/user/get')
      .set('accept-language', 'fr-CH;q=0.6, en;q=0.8, de;q=0.7, *;q=0.5');
    expect(res.header['set-cookie'][0]).toBe('light.lang=en; path=/; httponly');
    expect(i18n.lang).toBe('en');
  });

  // 解析accept-language *
  it('use accept-language *', async () => {
    const res = await util.request.get('/api/user/get')
      .set('accept-language', '*');
    expect(res.header['set-cookie'][0]).toBe('light.lang=zh; path=/; httponly');
    expect(i18n.lang).toBe('zh');
  });

  // 解析cookie
  it('use cookie', async () => {
    const res = await util.request.get('/api/user/get')
      .set('Cookie', ['light.lang=ja; path=/; httponly']);
    expect(res.header['set-cookie'][0]).toBe('light.lang=ja; path=/; httponly');
    expect(i18n.lang).toBe('ja');
  });

  // TODO: 解析session
  it('use session', async () => {
  });
});
