/**
 * middleware/router.test.js
 */

'use strict';

const Koa        = require('koa');
const middleware = require('../../../lib/middleware');
const timeout    = require('../../../lib/middleware/timeout');
const session    = require('../../../lib/middleware/session');

describe('Middleware', () => {

  beforeAll(() => {
    this.app = new Koa();
    this.app.context.db = { connection: jest.fn() };
  });

  afterEach(() => {
    jest.resetAllMocks();
  });

  /**
   * 确认加载middleware，通过custom修改默认设置
   */
  it('init', () => {
    const timeoutSpy = jest.spyOn(timeout, 'use');
    const sessionSpy = jest.spyOn(session, 'use');

    middleware.init(this.app, {
      timeout: { use: true, order: 500 },
      session: { use: false, order: 500 },
      none   : { use: true, name: 'none' }
    });

    expect(timeoutSpy).toHaveBeenCalled();
    expect(sessionSpy).not.toHaveBeenCalled();
  });

  /**
   * 确认加载middleware，指定一个完整的custom
   */
  it('init by custom', () => {
    const timeoutSpy = jest.spyOn(timeout, 'use');
    const sessionSpy = jest.spyOn(session, 'use');

    middleware.init(this.app, [
      { use: true, name: 'session' },
      { use: false, name: 'timeout' },
      { use: true, name: 'helloword' }
    ]);

    expect(timeoutSpy).not.toHaveBeenCalled();
    expect(sessionSpy).toHaveBeenCalled();
  });

});
