/**
 * middleware/method-override.test.js
 */

'use strict';

const util    = require('./util');
const user    = require('../../controllers/user');
const setting = require('../../../lib/setting');

describe('MethodOverride', () => {

  beforeAll(() => {
    setting.board = [{ api: '/api/user/remove', class: 'user', action: 'remove' }];
    util.setupKoa(['body-parse', 'method-override', 'router']);
  });

  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  it('header', async () => {
    const spy = jest.spyOn(user, 'remove');
    await util.request.post('/api/user/remove').set('X-HTTP-Method-Override', 'DELETE');
    await util.request.post('/api/user/remove').set('x-http-method', 'DELETE');
    await util.request.post('/api/user/remove').set('X-METHOD-OVERRIDE', 'DELETE');
    expect(spy.mock.calls[0][0].ctx.method).toBe('DELETE');
    expect(spy.mock.calls[1][0].ctx.method).toBe('DELETE');
    expect(spy.mock.calls[2][0].ctx.method).toBe('DELETE');
  });

  it('body', async () => {
    const spy = jest.spyOn(user, 'remove');
    await util.request.post('/api/user/remove').send({ _method: 'DELETE' });
    expect(spy.mock.calls[0][0].ctx.method).toBe('DELETE');
  });

  it('query', async () => {
    const spy = jest.spyOn(user, 'remove');
    await util.request.post('/api/user/remove?_method=delete');
    expect(spy.mock.calls[0][0].ctx.method).toBe('DELETE');
  });

  it('bad method', async () => {
    const spy = jest.spyOn(user, 'remove');
    await util.request.post('/api/user/remove');
    await util.request.post('/api/user/remove?_method=bad');
    expect(spy.mock.calls[0][0].ctx.method).toBe('POST');
    expect(spy.mock.calls[1][0].ctx.method).toBe('POST');
  });

  it('cannot override', async () => {
    const spy = jest.spyOn(user, 'remove');
    await util.request.put('/api/user/remove?_method=DELETE');
    expect(spy.mock.calls[0][0].ctx.method).toBe('PUT');
  });

});
