/**
 * middleware/query-parse.test.js
 */

'use strict';

const qs      = require('qs');
const util    = require('./util');
const user    = require('../../controllers/user');
const setting = require('../../../lib/setting');

describe('QueryParse', () => {

  beforeAll(() => {
    setting.board = [{ api: '/api/user/get', class: 'user', action: 'get' }];
    util.setupKoa(['query-parse', 'router']);
  });

  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  it('parse querystring', async () => {
    const spy = jest.spyOn(user, 'get');
    await util.request.get('/api/user/get?name=light');
    expect(spy.mock.calls[0][0].params).toEqual({ name: 'light' });
  });

  it('parse querystring object', async () => {
    const query = qs.stringify({ name: 'light', condition: { age: { $gt: '18' } } });
    const spy = jest.spyOn(user, 'get');
    await util.request.get(`/api/user/get?${query}`);
    expect(spy.mock.calls[0][0].params).toEqual({ name: 'light', condition: { age: { $gt: '18' } } });
  });
});
