/**
 * middleware/static.test.js
 */

'use strict';

const util = require('./util');
const config = require('../../../lib/configuration');

describe('Static', () => {

  beforeAll(() => util.setupKoa(['static', 'helloword']));
  afterAll(() => util.clear());

  // 解析favicon
  it('favicon', async () => {
    config.app = { static: '/test' };
    const res = await util.request.get('/favicon.ico');
    expect(res.type).toBe('image/vnd.microsoft.icon');
    expect(res.body).toHaveLength(67646);
  });

  // 解析静态文件
  it('static resources', async () => {
    config.app = { static: '/test' };
    const res = await util.request.get('/test/views/index.html');
    expect(res.type).toBe('text/html');
    expect(res.text).toContain('<script src="/static/js/light.js"></script>');
  });

  // 非静态文件路径
  it('non-static resources', async () => {
    const res = await util.request.get('/index.html');
    expect(res.type).toBe('text/plain');
    expect(res.text).toBe('Hello World');
  });

});
