/**
 * middleware/timeout.test.js
 */

'use strict';

const util  = require('./util');
const user    = require('../../controllers/user');
const config  = require('../../../lib/configuration');
const setting = require('../../../lib/setting');

describe('Timeout', () => {

  beforeAll(() => {
    setting.board = [{ api: '/api/user/get', class: 'user', action: 'get' }];
    util.setupKoa(['timeout', 'router']);
  });
  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  it('timeout', async () => {

    // 1ms timeout
    config.app = { timeout: 0.001 };

    // sleep 5ms
    jest.spyOn(user, 'get').mockImplementation(async () => await util.sleep(5));

    const res = await util.request.get('/api/user/get');
    expect(res.text).toBe('RPOCESS TIMEOUT');
    expect(res.status).toBe(500);
  });

  it('non timeout', async () => {
    config.app = { timeout: 0.005 };
    jest.spyOn(user, 'get').mockImplementation(async () => await util.sleep(1));
    const res = await util.request.get('/api/user/get');
    expect(res.status).toBe(200);
  });

  it('use special time (non-timeout)', async () => {

    config.app = { timeout: 0.001 };
    config.ignore = { timeout: ['^/api/user/.* 0.005'] };
    jest.spyOn(user, 'get').mockImplementation(async () => await util.sleep(3));

    const res = await util.request.get('/api/user/get');
    expect(res.status).toBe(200);
  });

  it('use special time (timeout)', async () => {

    config.app = { timeout: 0.005 };
    config.ignore = { timeout: ['^/api/user/.* 0.001'] };
    jest.spyOn(user, 'get').mockImplementation(async () => await util.sleep(3));

    const res = await util.request.get('/api/user/get');
    expect(res.text).toBe('RPOCESS TIMEOUT');
    expect(res.status).toBe(500);
  });

  it('ignore', async () => {

    config.app = { timeout: 0.001 };
    config.ignore = { timeout: ['^/api/user/.*'] };
    jest.spyOn(user, 'get').mockImplementation(async () => await util.sleep(5));

    const res = await util.request.get('/api/user/get');
    expect(res.status).toBe(200);
  });

});
