/**
 *
 */

'use strict';

const Koa        = require('koa');
const supertest  = require('supertest');
const middleware = require('../../../lib/middleware');

class Util {

  /**
   * 创建测试用koa实例，并使用指定中间件
   * 设置以下三个变量
   *  app
   *  server
   *  request
   *
   * @param keepcookie 设置为false，可以在每次it间不保持cookie
   */
  static setupKoa(use = [], keepcookie = true) {
    this.app = new Koa();

    // 初始化加载的中间件
    middleware.init(this.app, use.map(item => ({ name: item })));

    // 启动服务开始监听端口
    this.server = this.app.listen(process.env.APPPORT || 7000);

    // 是否每次都保持cookie
    if (keepcookie) {
      this.request = supertest.agent(this.server);
    } else {
      this.request = supertest(this.server);
    }
  }

  /**
   * 清除服务
   */
  static clear() {

    // 清除所有mock对象
    jest.resetAllMocks();

    // 关闭koa服务
    if (this.server) {
      this.server.close();
    }
  }

  /**
   * 模拟session store
   */
  static mockSessionStore() {

    this.store = {};
    jest.mock('../../../lib/db/sessionstore', () => ({
      init: () => ({
        get: async sid => (this.store[sid] || {}).data,
        set: async (sid, data, maxAge, { changed, rolling }) => {
          if (changed || rolling) {
            this.store[sid] = { sid, data, updatedAt: new Date() };
          }
          return data;
        },
        destroy: async id => delete this.store[id]
      })
    }));

  }

  /**
   * 等待
   * @param {*} time
   */
  static async sleep(time = 1000) {
    await new Promise(resolve => setTimeout(resolve, time));
  }

}

module.exports = Util;
