/**
 * middleware/xml-parse.test.js
 */

'use strict';

const util    = require('./util');
const user    = require('../../controllers/user');
const setting = require('../../../lib/setting');

describe('XmlParse', () => {

  beforeAll(() => {
    setting.board = [{ api: '/api/user/update', class: 'user', action: 'update' }];
    util.setupKoa(['xml-parse', 'router']);
  });

  afterAll(() => util.clear());
  afterEach(() => jest.resetAllMocks());

  it('post text/xml', async () => {
    const spy = jest.spyOn(user, 'update');
    await util.request.post('/api/user/update')
      .set('Content-Type', 'text/xml; charset=utf-8')
      .send('<root>Hello</root>');
    expect(spy.mock.calls[0][0].params).toEqual({ root: 'Hello' });
  });

  it('put application/xml', async () => {
    const spy = jest.spyOn(user, 'update');
    await util.request.put('/api/user/update')
      .set('Content-Type', 'application/xml')
      .send('<root>Hello</root>');
    expect(spy.mock.calls[0][0].params).toEqual({ root: 'Hello' });
  });

  it('patch text/xml', async () => {
    const spy = jest.spyOn(user, 'update');
    await util.request.patch('/api/user/update')
      .set('Content-Type', 'text/xml')
      .send('<root>Hello</root>');
    expect(spy.mock.calls[0][0].params).toEqual({ root: 'Hello' });
  });

  it('invalid method', async () => {
    const spy = jest.spyOn(user, 'update');
    await util.request.get('/api/user/update')
      .set('Content-Type', 'text/xml')
      .send('<root>Hello</root>');
    expect(spy.mock.calls[0][0].params).toEqual({});
  });

  it('invalid xml', async () => {
    const spy = jest.spyOn(user, 'update');
    await util.request.post('/api/user/update')
      .set('Content-Type', 'text/xml')
      .send('Hello');
    expect(spy.mock.calls[0][0].params).toEqual({});
  });

});
