/**
 * exporter.test.js
 */

'use strict';

const common   = require('../../../../lib/model/datamigrate/common');
const CONST    = require('../../../../lib/constant');
const setting  = require('../../../../lib/setting');
const Context  = require('../../../../lib/context');
const pool     = require('../../../../lib/db/mongo/pool');

describe('Exporter', () => {

  beforeAll(async () => {
    process.env.APPLOCAL = true;
    await setting.init();
  });

  afterAll(() => {
    pool.destroy(true);
  });

  it('join', async () => {

    // 导入时，名称转换为_id
    const handler = new Context(CONST.DEFAULT_USER);
    const map1 = {
      key       : 'type',
      schema    : 'test',
      conditions: { val: '$type' },
      fields    : ['_id']
    };

    handler.params.data = { type: 1, lang: [{ en: 1, ja: 1 }] };
    await common.join(handler, map1);
    expect(handler.params.data.type.length).toBeGreaterThan(0);
    expect(handler.params.data.lang).toEqual([{ en: 1, ja: 1 }]);

    // 导入时，一个字段有多个值，转换为相应的_id
    const map2 = {
      key       : 'lang',
      schema    : 'test',
      conditions: { val: '$lang' },
      fields    : ['_id']
    };

    handler.params.data = { lang: [1, 2] };
    await common.join(handler, map2);
    expect(handler.params.data.lang.length).toBeGreaterThan(0);

    // 倒出时，id转换成名称
    const map3 = {
      key       : 'lang.en',
      schema    : 'test',
      conditions: { val: '$lang.en' },
      fields    : ['createBy']
    };

    handler.params.data = { lang: [{ en: 1, ja: 1 }, { en: 2, ja: 2 }] };
    await common.join(handler, map3);
    expect(handler.params.data.lang.en.length).toBeGreaterThan(0);

  });

});
