/**
 * excel.test.js
 */

'use strict';

const os    = require('os');
const path  = require('path');
const fs    = require('fs');
const excel = require('../../../../lib/model/datamigrate/excel');

describe('Exporter', () => {

  it('dump', () => {
    const data = [];
    for (let idx = 0; idx < 10; idx += 1) {

      const row = [];
      for (let col = 0; col < 5; col += 1) {
        row.push(`r${idx}, c${col}`);
      }

      data.push(row);
    }

    const file = path.join(os.tmpdir(), 'excel.xlsx');
    excel.dump(file, data);
    expect(fs.existsSync(file)).toBe(true);
    expect(fs.statSync(file).size).toBe(16433);
  });

  it('parse', () => {

    // by index, no mapping
    const file = path.join(os.tmpdir(), 'excel.xlsx');
    const [res1] = excel.parse(file, [], 'index', true);
    expect(res1.title).toEqual(['r0, c0', 'r0, c1', 'r0, c2', 'r0, c3', 'r0, c4']);
    expect(res1.content.length).toBe(9);
    expect(res1.content[8]).toEqual({ 'r0, c0': 'r9, c0', 'r0, c1': 'r9, c1', 'r0, c2': 'r9, c2', 'r0, c3': 'r9, c3', 'r0, c4': 'r9, c4' });

    // by index, has mapping
    const map1 = [{ col: 1, key: 'col1' }, { col: 2, key: 'col2' }];
    const [res2] = excel.parse(file, map1, 'index', true);
    expect(res2.content[8]).toEqual({ col1: 'r9, c0', col2: 'r9, c1', 'r0, c2': 'r9, c2', 'r0, c3': 'r9, c3', 'r0, c4': 'r9, c4' });

    // by title
    const map2 = [
      { title: 'r0, c0', key: 'col1' },
      { title: 'r0, c1', key: 'col2', sanitize: {} },
      { title: 'r0, c2', key: 'col3', sanitize: { rule: 'fix' } },
      { title: 'r0, c3', key: 'col4' },
      { title: 'r0, c9', key: 'col5' }
    ];
    const [res3] = excel.parse(file, map2, 'title');
    expect(res3[0]).toEqual({ col1: 'r1, c0', col2: 'r1, c1', col3: 'r1, c2', col4: 'r1, c3' });

    fs.unlinkSync(file);

  });

});
