/**
 * security.test.js
 */

'use strict';

const security = require('../../lib/security');
const util = require('./middleware/util');

describe('Security', () => {

  beforeAll(() => {
    util.setupKoa(['session', 'csrf-check']);
  });

  afterAll(() => {
    util.clear();
  });

  it('simpleLogin', async () => {
    // TODO: 确认 renew session，renew后cookie，csrf正确
  });

  it('simpleLogout', () => {
    const handler = { ctx: { session: 1 } };
    security.simpleLogout(handler);
    expect(handler.ctx.session).toBe(null);
  });

  it('sha256', () => {
    expect(security.sha256('', 'light')).toBe('');
    expect(security.sha256('qazwsxed', 'light')).toBe('6124ddc4624922207e5aa14f62b1d628635f954594233e8cdda0ea733edc4cad');
  });

  it('encrypt', () => {
    expect(security.encrypt('qazwsxed', 'light')).toBe('21cae4c456f49e9af9369d981b971661');
  });

  it('decrypt', () => {
    expect(security.decrypt('21cae4c456f49e9af9369d981b971661', 'light')).toBe('qazwsxed');
  });

  it('encrypt2', () => {
    expect(security.encrypt2('qazwsxed', 'light')).toBe('B61UH+SzYBymgEcMQT31hA==');
  });

  it('decrypt2', () => {
    expect(security.decrypt2('SPvPSa3cTKdfLgE7hKh0Pw==', 'light')).toBe('2e35501c2b7e');
  });

});
